#include <LedControl.h>

#define VRX     A0
#define VRY     A1
#define SW      A2

#define CLK     13
#define CS      10
#define DIN     11

#define buttonPin 19

int buttonState = 0;
int ch1 = 5;
int ch2 = 8;
int ch3 = 9;
int ch4 = 4;
int aa = 6;
int bb = 12;
int cc = 2;
int dd = 1;
int ee = 0;
int ff = 7;
int gg = 3;
int snake[64][2];
int length;
int food[2];
int v[2];
bool game_over = false;

long current_time;
long prev_advance;
int blink_count;
const short messageSpeed = 3;
int score = 0;

LedControl lc = LedControl(DIN, CLK, CS, 1);

void init_game() {
    prev_advance = current_time = 0;
    blink_count = 3;
    score = 0;

    int half = 4;
    length = 2;
    
    for (int i = 0; i < length; i++) {
        snake[i][0] = half - 1;
        snake[i][1] = half + i;
    }
 
    food[0] = random(8);
    food[1] = random(8);

    v[0] = 0;
    v[1] = -1;

}

void render() {
    for (int i = 0; i < length; i++)
        lc.setLed(0, snake[i][0], snake[i][1], true);
    lc.setLed(0, food[0], food[1], true);

    if(score < 10) {
      score_part3();
      switch(score) {
      case 0: zero();
      break;
      
      case 1: one();
      break;

      case 2: two();
      break;

      case 3: three();
      break;

      case 4: four();
      break;

      case 5: five();
      break;

      case 6: six();
      break;

      case 7: seven();
      break;

      case 8: eight();
      break;

      case 9: nine();
      break;

      default: zero();
      break;
    }
    }
    else {
      int n = score / 10;
      int r = score % 10;
      score_part2();
      if(n == 1) {
        one();
        delay(2);
      }
      else
      {
        two();
        delay(2);
      }
      score_part3();
      switch(r) {
      case 0: zero();
      break;
      
      case 1: one();
      break;

      case 2: two();
      break;

      case 3: three();
      break;

      case 4: four();
      break;

      case 5: five();
      break;

      case 6: six();
      break;

      case 7: seven();
      break;

      case 8: eight();
      break;

      case 9: nine();
      break;

      default: zero();
      break;
    }
    } 
    
}

void clearScreen() {
    for (int x = 0; x < 8; x++) {
        for (int y = 0; y < 8; y++) {
            lc.setLed(0, x, y, false);
        }
    }
}

bool move_snake() {
    int head[2];
    head[0] = snake[0][0] + v[0];
    head[1] = snake[0][1] + v[1];

   if (head[0] < 0 || head[0] >= 8) {
        delay(400);
        showGameOverMessage();
        score_part3();
        none();
        return true;
    }
  
   if (head[1] < 0 || head[1] >= 8) {
        delay(400);
        score_part3();
        none();
        showGameOverMessage();
        return true;
    }

    for (int i = 0; i < length; i++) {
        if (snake[i][0] == head[0] && snake[i][1] == head[1]) {
            delay(400);
            score_part3();
            none();
            showGameOverMessage();
            return true;
        }
    }

    if (head[0] == food[0] && head[1] == food[1]) {
        length++;
        score++;  
        food[0] = random(8);
        food[1] = random(8);
    }

    for (int i = length - 1; i > 0; i--) {
        snake[i][0] = snake[i - 1][0];
        snake[i][1] = snake[i - 1][1];
    }

    snake[0][0] += v[0];
    snake[0][1] += v[1];

    return false;
}

void setup() {
    pinMode(SW, INPUT_PULLUP);
    pinMode(VRX, INPUT);
    pinMode(VRY, INPUT);

    pinMode(buttonPin, INPUT);

    attachInterrupt(digitalPinToInterrupt(SW), gameFunc, RISING);

    lc.shutdown(0, false);
    lc.setIntensity(0, 3);

    pinMode(ch1, OUTPUT);
    pinMode(ch2, OUTPUT);
    pinMode(ch3, OUTPUT);
    pinMode(ch4, OUTPUT);
    pinMode(aa, OUTPUT);
    pinMode(bb, OUTPUT);
    pinMode(cc, OUTPUT);
    pinMode(dd, OUTPUT);
    pinMode(ee, OUTPUT);
    pinMode(ff, OUTPUT);
    pinMode(gg, OUTPUT);
    
}

void loop() {
    buttonState = digitalRead(buttonPin);
    if (buttonState == HIGH) {
      gameFunc();
    }
    if (!game_over) {
        clearScreen();
        render();
        if (current_time - prev_advance > 15) {
            game_over = move_snake();
            prev_advance = current_time;    
        }

        else {
            while (blink_count > 0) {
                clearScreen();
                delay(300);
                render();
                delay(300);
                blink_count--;
            }
        }

        readControls();
        current_time++;
    } 
    
}

void gameFunc() {  
    if(game_over == true){
        game_over = false;
        init_game();
    }
}

void readControls() {
    int xValue = analogRead(VRX);
    int yValue = analogRead(VRY);
 
    int dx = map(xValue, 0, 906, 1, -1);
    int dy = map(yValue, 0, 906, -1, 1);

    if (dy != 0 && v[0] != 0) {
        v[0] = 0;
        v[1] = dy;    
    }
   
    if (dx != 0 && v[1] != 0) {
        v[0] = dx;
        v[1] = 0;
    }
}

void none() {
  digitalWrite(aa,1);
  digitalWrite(bb,1);
  digitalWrite(cc,1);
  digitalWrite(dd,1);
  digitalWrite(ee,1);
  digitalWrite(ff,1);
  digitalWrite(gg,0);
}

void zero() {
  digitalWrite(aa,0);
  digitalWrite(bb,0);
  digitalWrite(cc,0);
  digitalWrite(dd,0);
  digitalWrite(ee,0);
  digitalWrite(ff,0);
  digitalWrite(gg,1);
}

void one() {
  digitalWrite(aa,1);
  digitalWrite(bb,0);
  digitalWrite(cc,0);
  digitalWrite(dd,1);
  digitalWrite(ee,1);
  digitalWrite(ff,1);
  digitalWrite(gg,1);
}

void two() {
  digitalWrite(aa,0);
  digitalWrite(bb,0);
  digitalWrite(cc,1);
  digitalWrite(dd,0);
  digitalWrite(ee,0);
  digitalWrite(ff,1);
  digitalWrite(gg,0);
}

void three() {
  digitalWrite(aa,0);
  digitalWrite(bb,0);
  digitalWrite(cc,0);
  digitalWrite(dd,0);
  digitalWrite(ee,1);
  digitalWrite(ff,1);
  digitalWrite(gg,0);
}

void four() {
  digitalWrite(aa,1);
  digitalWrite(bb,0);
  digitalWrite(cc,0);
  digitalWrite(dd,1);
  digitalWrite(ee,1);
  digitalWrite(ff,0);
  digitalWrite(gg,0);
}

void five() {
  digitalWrite(aa,0);
  digitalWrite(bb,1);
  digitalWrite(cc,0);
  digitalWrite(dd,0);
  digitalWrite(ee,1);
  digitalWrite(ff,0);
  digitalWrite(gg,0);
}

void six() {
  digitalWrite(aa,0);
  digitalWrite(bb,1);
  digitalWrite(cc,0);
  digitalWrite(dd,0);
  digitalWrite(ee,0);
  digitalWrite(ff,0);
  digitalWrite(gg,0);
}

void seven() {
  digitalWrite(aa,0);
  digitalWrite(bb,0);
  digitalWrite(cc,0);
  digitalWrite(dd,1);
  digitalWrite(ee,1);
  digitalWrite(ff,1);
  digitalWrite(gg,1);
}

void eight() {
  digitalWrite(aa,0);
  digitalWrite(bb,0);
  digitalWrite(cc,0);
  digitalWrite(dd,0);
  digitalWrite(ee,0);
  digitalWrite(ff,0);
  digitalWrite(gg,0);
}

void nine() {
  digitalWrite(aa,0);
  digitalWrite(bb,0);
  digitalWrite(cc,0);
  digitalWrite(dd,0);
  digitalWrite(ee,1);
  digitalWrite(ff,0);
  digitalWrite(gg,0);
}

void score_part2() {
  digitalWrite(ch1,LOW);
  digitalWrite(ch2,HIGH);
  digitalWrite(ch3,LOW);
  digitalWrite(ch4,LOW);
}

void score_part3() {
  digitalWrite(ch1,LOW);
  digitalWrite(ch2,LOW);
  digitalWrite(ch3,HIGH);
  digitalWrite(ch4,LOW);
}

const PROGMEM bool gameOverMessage[8][90] = {
  {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,1,0,0,0,1,1,0,0,0,1,1,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1,1,0,0,1,1,0,0,1,1,1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,1,0,1,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,1,1,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,1,1,1,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,1,1,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,1,1,0,1,0,1,1,0,0,1,1,1,1,1,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,1,1,1,0,0,0,1,1,1,1,1,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,1,1,0,1,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,0,1,1,1,1,0,0,0,1,1,0,0,0,0,0,0,1,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,1,1,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,1,1,0,0,0,0,1,1,1,1,1,1,0,0,1,1,0,0,1,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0}
}; 


void showGameOverMessage() {
    for (int d = 0; d < sizeof(gameOverMessage[0]) - 7; d++) {
        for (int col = 0; col < 8; col++) {
            delay(messageSpeed);
            for (int row = 0; row < 8; row++) {
                lc.setLed(0, row, col, pgm_read_byte(&(gameOverMessage[row][col + d])));
            }
        }
    }
}
